#include "config.h"
// threading library
#include "pt_cornell_1_2_1.h"

////////////////////////////////////
// graphics libraries
#include "tft_master.h"
#include "tft_gfx.h"
// need for rand function
#include <stdlib.h>
////////////////////////////////////

// string output buffer
char buffer[60];

// === thread structures ============================================
// thread control structs
static struct pt pt_adc ;

static PT_THREAD (protothread_adc(struct pt *pt)) {
    PT_BEGIN(pt);
    static int adc_1;
    static int adc_5;

    static int adc_5_holder[4];
    static int adc_1_holder[4];
    static int adc_5_ptr = 0;
    static int adc_1_ptr = 0;
    static int tmp_adc_5;
    static int tmp_adc_1;
    while(1) {
        // yield time 50ms
        PT_YIELD_TIME_msec(50);

        // read the ADC AN1
        // the x_out value from accelerometer
        adc_1 = ReadADC10(0);
        adc_1_holder[adc_1_ptr++ % 4] = adc_1; // store valule into array
        tmp_adc_1 = 0;
        int i;
        for (i = 0; i < 4; i++) { // calculate average value
            tmp_adc_1 += adc_1_holder[i]/4;
        }
        adc_1 = tmp_adc_1; // assign averaged value as input
        if (adc_1 < 490) {
            // move motor to desired direction
            motor_left_forward();
        } else if (adc_1 > 500) {
            // move motor to desired direction
            motor_left_back();
        } else {
            // in steady state, hold
            motor_left_holds();     
        }

        // read from AN5
        adc_5 = ReadADC10(1);
        // store value into array
        adc_5_holder[adc_5_ptr++ % 4] = adc_5;
        tmp_adc_5 = 0;
        // calculate average value
        for (i = 0; i < 4; i++) {
          tmp_adc_5 += adc_5_holder[i]/4;
        }
        // use averaged value as input
        adc_5 = tmp_adc_5;
        if (adc_5 < 490) {
          // move motor to desired direction
            motor_right_back();
        } else if (adc_5 > 510) {
            // move motor to desired direction
            motor_right_forward();
        } else {
            // in steady state, hold
            motor_right_holds();
        }


        // print ADC value onto svreen
        tft_fillRoundRect(0,100, 230, 15, 1, ILI9340_BLACK);// x,y,w,h,radius,color
        tft_setCursor(0, 100);
        tft_setTextColor(ILI9340_YELLOW);
        tft_setTextSize(2);
        sprintf(buffer,"%d %d", adc_1, adc_5);
        tft_writeString(buffer);
    } // END WHILE(1)
    PT_END(pt);
} // animation thread

// === Main  ======================================================
void main(void) {
    ANSELA = 0; ANSELB = 0; 

    // === config threads ==========
    // turns OFF UART support and debugger pin, unless defines are set
    PT_setup();

    // === setup system wide interrupts  ========
    INTEnableSystemMultiVectoredInt();
    CloseADC10(); // ensure the ADC is off before setting the configuration

    // define setup parameters for OpenADC10
    // Turn module on | ouput in integer | trigger mode auto | enable autosample
    // ADC_CLK_AUTO -- Internal counter ends sampling and starts conversion (Auto convert)
    // ADC_AUTO_SAMPLING_ON -- Sampling begins immediately after last conversion completes; SAMP bit is automatically set
    // ADC_AUTO_SAMPLING_OFF -- Sampling begins with AcquireADC10();
    #define PARAM1  ADC_FORMAT_INTG16 | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_ON //

    // define setup parameters for OpenADC10
    // ADC ref external  | disable offset test | disable scan mode | do 2 sample | use single buf | alternate mode on
    #define PARAM2  ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_OFF | ADC_SAMPLES_PER_INT_2 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_ON
    //
    // Define setup parameters for OpenADC10
    // use peripherial bus clock | set sample time | set ADC clock divider
    // ADC_CONV_CLK_Tcy2 means divide CLK_PB by 2 (max speed)
    // ADC_SAMPLE_TIME_5 seems to work with a source resistance < 1kohm
    // SLOW it down a little
    #define PARAM3 ADC_CONV_CLK_PB | ADC_SAMPLE_TIME_15 | ADC_CONV_CLK_Tcy //ADC_SAMPLE_TIME_15| ADC_CONV_CLK_Tcy2

    // define setup parameters for OpenADC10
    // set AN11 and  as analog inputs
    #define PARAM4 ENABLE_AN1_ANA | ENABLE_AN5_ANA // 

    // define setup parameters for OpenADC10
    // do not assign channels to scan
    #define PARAM5 SKIP_SCAN_ALL //|SKIP_SCAN_AN5 //SKIP_SCAN_AN1 |SKIP_SCAN_AN5  //SKIP_SCAN_ALL

    // configure to sample AN5 and AN1 on MUX A and B
    SetChanADC10(ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN1 | ADC_CH0_NEG_SAMPLEB_NVREF | ADC_CH0_POS_SAMPLEB_AN5);

    OpenADC10(PARAM1, PARAM2, PARAM3, PARAM4, PARAM5 ); // configure ADC using the parameters defined above

    EnableADC10(); // Enable the ADC

    // init control thread
    PT_INIT(&pt_adc);

    // init the display
    tft_init_hw();
    tft_begin();
    tft_fillScreen(ILI9340_BLACK);
    //240x320 vertical display
    tft_setRotation(0); // Use tft_setRotation(1) for 320x240

    // setup motors
    setup_motors();

    // there is only one thread to schedule
    while (1){
      PT_SCHEDULE(protothread_adc(&pt_adc));
    }
} // main

// === end  ======================================================

